
# q-expansion of newform 855.2.k.h, downloaded from the LMFDB on 14 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 855
weight = 2
poly_data = [9, 12, 31, -14, 26, -3, 6, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0, 0, 0], 1], [[-1215, 14317, -3618, 3051, -911, 729, -189, -26], 4243], [[1947, 483, 989, 3314, 276, 529, 20, 115], 4243], [[-11694, -567, -1161, -2599, -324, -621, 161, -135], 4243], [[6390, -17152, 19028, -16046, 3534, -3834, 994, -649], 12729], [[6573, 16249, 338, 12064, 193, 2845, -109, 434], 4243], [[-5355, 16585, -15946, 13447, -3858, 3213, -833, 514], 4243]]

hecke_ring_character_values = [[191, [1, 0, 0, 0, 0, 0, 0, 0]], [172, [1, 0, 0, 0, 0, 0, 0, 0]], [496, [-1, 0, 0, 0, 0, 1, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0, 0, -1], [0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 1, 0, 0], [-1, 0, 1, 0, 0, 0, -1, 0], [0, 0, -1, -2, 0, 0, 1, 0], [-2, -1, 0, -1, 0, 2, 1, 0], [0, -2, 0, 0, 0, 0, 0, -1], [1, -1, 2, 0, 1, 0, 0, 0], [0, -1, 0, -1, -1, 0, -1, 1], [0, -1, 0, -1, 2, 0, -2, -2], [-1, 0, 1, -1, -3, 0, -1, 0], [-1, 0, -1, 0, -2, 0, 1, 0], [0, 2, 3, 0, 0, -3, 0, -2], [0, -4, -1, 0, 0, 1, 0, 1], [-3, -2, 0, -2, 2, 3, 3, -2], [0, 2, 0, 2, 3, 0, -2, -3], [0, -5, 0, 0, 0, 0, 0, 0], [1, 0, 0, 0, 4, -1, 2, -4], [-2, -2, 0, -2, -2, 2, 0, 2], [0, -1, 0, 0, 0, 6, 0, 3], [0, 3, -1, 0, 0, 4, 0, -1], [0, 1, 1, 0, 0, -5, 0, -2], [0, 0, -2, 2, -1, 0, 2, 0], [3, 3, 0, 3, -2, -3, -3, 2], [0, -5, -4, 0, 0, 1, 0, 0], [-3, 1, 0, 1, -1, 3, -3, 1], [2, 0, -1, -3, 1, 0, 1, 0], [-6, 0, 1, 1, -5, 0, -1, 0], [0, -1, -2, 0, 0, -8, 0, 1], [3, 0, -2, -3, 2, 0, 2, 0], [-8, 2, 0, 2, 2, 8, 2, -2], [0, 0, -1, 0, 0, -3, 0, -8], [-3, -4, 0, -4, 1, 3, 7, -1], [4, -5, 0, -5, -4, -4, -1, 4], [0, 0, -4, 0, 0, -3, 0, 7], [5, 0, -4, -4, -4, 0, 4, 0], [0, -1, -2, 0, 0, 1, 0, 1], [2, 0, -6, 4, -1, 0, 6, 0], [-3, -6, 0, -6, 0, 3, 3, 0], [0, 6, -7, 0, 0, -3, 0, -4], [3, 0, 4, -4, 5, 0, -4, 0], [10, 7, 0, 7, -2, -10, -1, 2], [-3, 0, -5, -1, 4, 0, 5, 0], [0, -4, 2, 0, 0, -8, 0, 3], [15, 0, 3, -2, 6, 0, -3, 0], [-2, 3, 0, 3, 3, 2, 4, -3], [0, 1, 7, 0, 0, -11, 0, -2], [0, 1, -6, 0, 0, -11, 0, -6], [-18, 0, -2, -2, 4, 0, 2, 0], [-4, 0, 1, -4, 2, 0, -1, 0], [0, -5, -6, 0, 0, 9, 0, -1], [9, 0, -1, 9, -3, 0, 1, 0], [-5, -3, 0, -3, -4, 5, -7, 4], [0, -4, 0, -4, -8, 0, -1, 8], [-3, 1, 0, 1, -2, 3, 6, 2], [0, -2, 4, 0, 0, 0, 0, 0], [0, 6, -4, 0, 0, -3, 0, -6], [0, 0, 6, 0, 0, 13, 0, 5], [8, 0, -6, -2, -2, 0, 6, 0], [-9, 7, 0, 7, -3, 9, 6, 3], [0, 9, -4, 0, 0, -5, 0, -5], [12, 0, -5, -5, -3, 0, 5, 0], [0, 3, 4, 0, 0, -11, 0, -4], [9, 0, 1, -2, 2, 0, -1, 0], [-14, -1, 0, -1, -7, 14, -1, 7], [-3, 2, 0, 2, -3, 3, -9, 3], [5, 0, -2, 13, 0, 0, 2, 0], [0, 14, 4, 0, 0, 4, 0, 2], [0, -2, 3, 0, 0, -3, 0, 5], [5, 0, 6, -3, 9, 0, -6, 0], [0, 0, -6, 4, 5, 0, 6, 0], [0, 4, 6, 0, 0, 0, 0, 5], [-8, 3, 0, 3, -6, 8, 7, 6], [11, 0, 5, 6, -9, 0, -5, 0], [8, 0, -2, -5, -3, 0, 2, 0], [0, -2, -1, 0, 0, -3, 0, -3], [12, -8, 0, -8, 0, -12, 5, 0], [0, -8, 3, 0, 0, 13, 0, 0], [0, -9, 6, 0, 0, 6, 0, 3], [13, 0, 0, 0, 1, -13, -2, -1], [-3, 0, 3, 1, 5, 0, -3, 0], [0, -14, -7, 0, 0, 4, 0, -2], [-18, 3, 0, 3, -4, 18, -4, 4], [7, -1, 0, -1, -4, -7, -2, 4], [0, -6, 1, 0, 0, 10, 0, 4], [-15, 4, 0, 4, 0, 15, 5, 0], [9, 0, 2, 4, -7, 0, -2, 0], [-10, 0, 4, -4, 5, 0, -4, 0], [0, -3, 2, 0, 0, -6, 0, 4], [20, 0, 3, 11, -7, 0, -3, 0], [18, 0, -9, 9, 0, 0, 9, 0], [6, -12, 0, -12, 7, -6, -3, -7], [14, 0, 0, -12, -5, 0, 0, 0], [0, 0, 4, 0, 0, 3, 0, -5], [0, 4, 9, 0, 0, 13, 0, 7], [-9, 7, 0, 7, 8, 9, -2, -8], [-18, 0, 0, 0, 1, 18, 6, -1], [12, 0, 5, 2, 1, 0, -5, 0], [-5, -7, 0, -7, 10, 5, -2, -10], [-32, -3, 0, -3, -5, 32, -6, 5], [-8, 2, 0, 2, -5, 8, 6, 5], [15, 6, 0, 6, 3, -15, -11, -3], [6, 0, 1, -13, 2, 0, -1, 0], [-3, 0, 1, -13, -2, 0, -1, 0], [-7, 0, -2, -2, 6, 0, 2, 0], [2, 0, 8, 10, -1, 0, -8, 0], [0, 9, 0, 0, 0, -9, 0, -1], [-6, 4, 0, 4, -2, 6, -10, 2], [6, 5, 0, 5, -8, -6, -1, 8], [5, 0, -1, -3, -8, 0, 1, 0], [14, 0, 7, 1, 7, 0, -7, 0], [0, 13, -3, 0, 0, -20, 0, 5], [-6, -3, 0, -3, -5, 6, -1, 5], [-19, 0, 0, -9, -2, 0, 0, 0], [-17, -9, 0, -9, -12, 17, -9, 12], [0, -3, 9, 0, 0, -9, 0, 5], [0, -4, 9, 0, 0, 7, 0, 10], [-9, 0, -1, 4, 11, 0, 1, 0], [-3, 0, 0, 1, 2, 0, 0, 0], [-18, -9, 0, -9, 11, 18, 3, -11], [10, -1, 0, -1, -2, -10, 6, 2], [23, 0, -10, 3, 10, 0, 10, 0], [3, 0, 9, 6, 5, 0, -9, 0], [-9, 0, -2, -9, -10, 0, 2, 0], [14, 0, 12, 9, 2, 0, -12, 0], [0, 3, 2, 0, 0, 3, 0, 2], [-2, 7, 0, 7, 0, 2, -14, 0], [0, -6, 0, 0, 0, -3, 0, 8], [0, 2, -5, 0, 0, 7, 0, -5], [-7, 0, -10, -5, -2, 0, 10, 0], [0, -17, 1, 0, 0, 1, 0, 1], [0, -4, 1, 0, 0, 3, 0, -1], [4, 11, 0, 11, 4, -4, 0, -4], [0, -3, -9, 0, 0, 10, 0, 0], [-9, 0, 7, -2, -1, 0, -7, 0], [1, 1, 0, 1, 23, -1, 9, -23], [21, -7, 0, -7, 11, -21, -4, -11], [2, 0, 14, -6, 7, 0, -14, 0], [15, 0, 16, 11, -4, 0, -16, 0], [-21, 0, 9, -11, -9, 0, -9, 0], [1, -12, 0, -12, 0, -1, 14, 0], [-6, 14, 0, 14, 9, 6, 1, -9], [4, 4, 0, 4, 16, -4, 12, -16], [3, -7, 0, -7, -18, -3, 0, 18], [11, 0, -3, 0, 19, 0, 3, 0], [0, 0, 8, 0, 0, -30, 0, -7], [0, -9, -1, 0, 0, 4, 0, -17], [0, -2, -15, 0, 0, 3, 0, 2], [4, 11, 0, 11, 1, -4, 6, -1], [-9, 0, -1, 14, 9, 0, 1, 0], [0, 2, -13, 0, 0, -11, 0, 8], [15, 0, 5, -1, 14, 0, -5, 0], [13, -10, 0, -10, -9, -13, -3, 9], [3, -7, 0, -7, 3, -3, 14, -3], [0, -10, -6, 0, 0, -8, 0, 4], [21, 0, -2, 12, 4, 0, 2, 0], [17, 0, 0, 1, 7, 0, 0, 0], [0, -15, 0, 0, 0, 6, 0, 13], [-11, -9, 0, -9, 9, 11, -8, -9], [-27, -11, 0, -11, -8, 27, -1, 8], [0, -9, -2, 0, 0, 3, 0, -7], [0, 7, -2, 0, 0, 21, 0, 2], [0, 4, -4, 0, 0, -14, 0, 8], [0, -7, 8, 0, 0, 3, 0, -9], [6, 0, -3, -1, 3, 0, 3, 0], [0, 6, -3, 0, 0, 15, 0, 7], [0, 4, 10, 0, 0, -14, 0, -16], [4, 13, 0, 13, 10, -4, -3, -10]]
